@@ -1,6 +1,6 @@
 // -*- mode: C++ -*-
 //
-// Copyright (C) 2013-2015 Red Hat, Inc.
+// Copyright (C) 2013-2017 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -31,10 +31,6 @@ namespace abigail
 namespace ir
 {
 
-class corpus;
-/// A convenience typedef for shared pointer to @ref corpus.
-typedef shared_ptr<corpus> corpus_sptr;
-
 /// This is the abstraction of a set of translation units (themselves
 /// seen as bundles of unitary abi artefacts like types and decls)
 /// bundled together as a corpus.  A corpus is thus the Application
@@ -72,13 +68,24 @@ public:
   };
 
 private:
-  shared_ptr<priv> priv_;
-
   corpus();
 
-public:
+  void record_canonical_type(const type_base_sptr&) const;
+  type_base_sptr lookup_canonical_type(const string& qualified_name) const;
 
-  corpus(const string&);
+public:
+  shared_ptr<priv> priv_;
+
+  corpus(ir::environment*, const string& path= "");
+
+  const environment*
+  get_environment() const;
+
+  environment*
+  get_environment();
+
+  void
+  set_environment(environment*) const;
 
   void
   add(const translation_unit_sptr);
@@ -86,9 +93,24 @@ public:
   const translation_units&
   get_translation_units() const;
 
+  const translation_unit_sptr
+  find_translation_unit(const string &path) const;
+
   void
   drop_translation_units();
 
+  type_maps&
+  get_types();
+
+  const type_maps&
+  get_types() const;
+
+  type_maps&
+  get_type_per_loc_map();
+
+  const type_maps&
+  get_type_per_loc_map() const;
+
   origin
   get_origin() const;
 
@@ -114,12 +136,12 @@ public:
   set_soname(const string&);
 
   const string&
-  get_architecture_name();
+  get_architecture_name() const;
 
   void
   set_architecture_name(const string&);
 
-  bool
+  virtual bool
   is_empty() const;
 
   bool
@@ -149,7 +171,7 @@ public:
   const string_elf_symbols_map_type&
   get_undefined_fun_symbol_map() const;
 
-  const elf_symbols&
+  virtual const elf_symbols&
   get_sorted_fun_symbols() const;
 
   const elf_symbols&
@@ -167,7 +189,7 @@ public:
   const string_elf_symbols_map_type&
   get_undefined_var_symbol_map() const;
 
-  const elf_symbols&
+  virtual const elf_symbols&
   get_sorted_var_symbols() const;
 
   const elf_symbols&
@@ -178,31 +200,40 @@ public:
 
   const elf_symbol_sptr
   lookup_function_symbol(const string& symbol_name,
-			 const string& symbol_version) const;
+			 const elf_symbol::version& version) const;
+
+  const elf_symbol_sptr
+  lookup_function_symbol(const elf_symbol& symbol) const;
 
   const elf_symbol_sptr
   lookup_variable_symbol(const string& n) const;
 
   const elf_symbol_sptr
   lookup_variable_symbol(const string& symbol_name,
-			 const string& symbol_version) const;
+			 const elf_symbol::version& version) const;
 
-  const functions&
+  const elf_symbol_sptr
+  lookup_variable_symbol(const elf_symbol& symbol) const;
+
+  virtual const functions&
   get_functions() const;
 
+  const vector<function_decl*>*
+  lookup_functions(const string& id) const;
+
   void
   sort_functions();
 
-  const variables&
+  virtual const variables&
   get_variables() const;
 
   void
   sort_variables();
 
-  const elf_symbols&
+  virtual const elf_symbols&
   get_unreferenced_function_symbols() const;
 
-  const elf_symbols&
+  virtual const elf_symbols&
   get_unreferenced_variable_symbols() const;
 
   vector<string>&
@@ -246,6 +277,8 @@ public:
 
   exported_decls_builder_sptr
   get_exported_decls_builder() const;
+
+  friend class type_base;
 };// end class corpus.
 
 /// Abstracts the building of the set of exported variables and
@@ -261,12 +294,6 @@ public:
 
   /// Convenience typedef for shared_ptr<priv>
   typedef shared_ptr<priv> priv_sptr;
-  /// Convenience typedef for a hash map which key is a string an
-  /// which data is an abigail::ir::function_decl*
-  typedef unordered_map<string, function_decl*> str_fn_ptr_map_type;
-  /// Convenience typedef for a hash map which key is a string and
-  /// which data is an abigail::ir::var_decl*.
-  typedef unordered_map<string, var_decl*> str_var_ptr_map_type;
 
   friend class corpus;
 
@@ -307,6 +334,65 @@ public:
   maybe_add_var_to_exported_vars(var_decl*);
 }; //corpus::exported_decls_builder
 
+/// Abstraction of a group of corpora.
+///
+/// A corpus group is a union of corpora.  It provides a unified view
+/// of a set of corpora.  It lets you get the set of functions,
+/// variables and symbols that are defined and exported by a set of
+/// corpora.
+class corpus_group : public corpus
+{
+  struct priv;
+  typedef shared_ptr<priv> priv_sptr;
+
+  priv_sptr priv_;
+
+  // Forbid copy
+  corpus_group(const corpus_group&);
+
+public:
+  typedef vector<corpus_sptr> corpora_type;
+
+  corpus_group(ir::environment*, const string&);
+
+  virtual ~corpus_group();
+
+  void add_corpus(const corpus_sptr&);
+
+  const corpora_type&
+  get_corpora() const;
+
+  virtual bool
+  is_empty() const;
+
+  virtual const corpus::functions&
+  get_functions() const;
+
+  virtual const corpus::variables&
+  get_variables() const;
+
+  const string_elf_symbols_map_type&
+  get_var_symbol_map() const;
+
+  const string_elf_symbols_map_type&
+  get_fun_symbol_map() const;
+
+  virtual const elf_symbols&
+  get_sorted_fun_symbols() const;
+
+  virtual const elf_symbols&
+  get_sorted_var_symbols() const;
+
+  virtual const elf_symbols&
+  get_unreferenced_function_symbols() const;
+
+  virtual const elf_symbols&
+  get_unreferenced_variable_symbols() const;
+
+  bool
+  operator==(const corpus_group&) const;
+}; // end class corpus_group
+
 }// end namespace ir
 }//end namespace abigail
 #endif //__ABG_CORPUS_H__
