@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2015 Red Hat, Inc.
+// Copyright (C) 2013-2016 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -25,6 +25,7 @@
 #include <ostream>
 #include <istream>
 #include <iostream>
+#include <abg-suppression.h>
 
 namespace abigail
 {
@@ -38,17 +39,61 @@ using std::ifstream;
 using std::string;
 using std::tr1::shared_ptr;
 
+const char* get_system_libdir();
 bool file_exists(const string&);
 bool is_regular_file(const string&);
 bool is_dir(const string&);
+bool dir_exists(const string&);
+bool dir_is_empty(const string &);
+bool maybe_get_symlink_target_file_path(const string& file_path,
+					string& target_path);
 bool base_name(string const& path,
 	       string& file_name);
 bool dir_name(string const &path,
 	      string& path_dir_name);
 bool ensure_dir_path_created(const string&);
 bool ensure_parent_dir_created(const string&);
-bool check_file(const string& path, ostream& out);
+ostream& emit_prefix(const string& prog_name, ostream& out);
+bool check_file(const string& path, ostream& out, const string& prog_name = "");
+bool check_dir(const string& path, ostream& out, const string& prog_name="");
 bool string_ends_with(const string&, const string&);
+bool string_begins_with(const string&, const string&);
+bool string_is_ascii(const string&);
+bool string_is_ascii_identifier(const string&);
+
+suppr::type_suppression_sptr
+gen_suppr_spec_from_headers(const string& hdrs_root_dir);
+
+bool
+gen_suppr_spec_from_kernel_abi_whitelist(const string& abi_whitelist_path,
+					 suppr::suppressions_type& s);
+
+bool
+get_vmlinux_path_from_kernel_dist(const string&	from,
+				  string&		vmlinux_path);
+
+bool
+get_binary_paths_from_kernel_dist(const string&	dist_root,
+				  const string&	debug_info_root_path,
+				  string&		vmlinux_path,
+				  vector<string>&	module_paths);
+
+bool
+get_binary_paths_from_kernel_dist(const string&	dist_root,
+				  string&		vmlinux_path,
+				  vector<string>&	module_paths);
+
+string
+get_default_system_suppression_file_path();
+
+string
+get_default_user_suppression_file_path();
+
+void
+load_default_system_suppressions(suppr::suppressions_type&);
+
+void
+load_default_user_suppressions(suppr::suppressions_type&);
 
 class temp_file;
 
@@ -108,9 +153,12 @@ enum file_type
   FILE_TYPE_ELF,
   /// An archive (AR) file.
   FILE_TYPE_AR,
-  // A native xml file format representing a corpus of one or several
-  // translation units.
+  // A native abixml file format representing a corpus of one or
+  // several translation units.
   FILE_TYPE_XML_CORPUS,
+  // A native abixml file format representing a corpus group of one or
+  // several corpora.
+  FILE_TYPE_XML_CORPUS_GROUP,
   // A zip file, possibly containing a corpus of one of several
   // translation units.
   FILE_TYPE_ZIP_CORPUS,
@@ -129,7 +177,7 @@ enum file_type
 
 /// Exit status for abidiff and abicompat tools.
 ///
-/// It's actually a bit mask.  The valu of each enumerator is a power
+/// It's actually a bit mask.  The value of each enumerator is a power
 /// of two.
 enum abidiff_status
 {
@@ -169,6 +217,7 @@ operator&(abidiff_status, abidiff_status);
 
 abidiff_status&
 operator|=(abidiff_status&l, abidiff_status r);
+
 bool
 abidiff_status_has_error(abidiff_status s);
 
@@ -182,10 +231,51 @@ ostream&
 operator<<(ostream& output, file_type r);
 
 file_type guess_file_type(istream& in);
+
 file_type guess_file_type(const string& file_path);
 
+bool
+get_rpm_name(const string& str, string& name);
+
+bool
+get_rpm_arch(const string& str, string& arch);
+
+bool
+get_deb_name(const string& str, string& name);
+
+bool
+file_is_kernel_package(const string& file_path,
+		       file_type file_type);
+
+bool
+file_is_kernel_debuginfo_package(const string& file_path,
+				 file_type file_type);
+
 std::tr1::shared_ptr<char>
 make_path_absolute(const char*p);
 
+corpus_group_sptr
+build_corpus_group_from_kernel_dist_under(const string&	root,
+					  const string		debug_info_root,
+					  const string&	vmlinux_path,
+					  vector<string>&	suppr_paths,
+					  vector<string>&	kabi_wl_paths,
+					  suppr::suppressions_type&	supprs,
+					  bool				verbose,
+					  environment_sptr&		env);
+
+extern const char* PRIVATE_TYPES_SUPPR_SPEC_NAME;
 }// end namespace tools_utils
+
+/// A macro that expands to aborting the program when executed.
+///
+/// Before aborting, the macro emits informatin about the source
+/// location where it was expanded.
+#define ABG_ASSERT_NOT_REACHED \
+  do {									\
+    std::cerr << "in " << __FUNCTION__					\
+	      << " at: " << __FILE__ << ":" << __LINE__			\
+	      << ": execution should not have reached this point!\n";	\
+      abort();								\
+  } while (false)
 }//end namespace abigail
