@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2015 Red Hat, Inc.
+// Copyright (C) 2013-2017 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -25,8 +25,15 @@
 /// This file contains definitions of diff objects filtering
 /// facilities.
 
+#include "abg-internal.h"
+// <headers defining libabigail's API go under here>
+ABG_BEGIN_EXPORT_DECLARATIONS
+
 #include "abg-comp-filter.h"
 
+ABG_END_EXPORT_DECLARATIONS
+// </headers defining libabigail's API>
+
 namespace abigail
 {
 namespace comparison
@@ -185,7 +192,7 @@ has_type_size_change(const diff* diff)
 ///
 /// @return true iff the access specifier changed.
 static bool
-access_changed(decl_base_sptr f, decl_base_sptr s)
+access_changed(const decl_base_sptr& f, const decl_base_sptr& s)
 {
   if (!is_member_decl(f)
       || !is_member_decl(s))
@@ -212,8 +219,8 @@ access_changed(decl_base_sptr f, decl_base_sptr s)
 ///
 /// @return true if the test is positive, false otherwise.
 static bool
-function_name_changed_but_not_symbol(const function_decl_sptr f,
-				     const function_decl_sptr s)
+function_name_changed_but_not_symbol(const function_decl_sptr& f,
+				     const function_decl_sptr& s)
 {
   if (!f || !s)
     return false;
@@ -225,8 +232,8 @@ function_name_changed_but_not_symbol(const function_decl_sptr f,
       elf_symbol_sptr fs = f->get_symbol(), ss = s->get_symbol();
       if (fs == ss)
 	return true;
-      for (elf_symbol* s = fs->get_next_alias();
-	   s && s != fs->get_main_symbol();
+      for (elf_symbol_sptr s = fs->get_next_alias();
+	   s && !s->is_main_symbol();
 	   s = s->get_next_alias())
 	if (*s == *ss)
 	  return true;
@@ -287,7 +294,8 @@ data_member_offset_changed(decl_base_sptr f, decl_base_sptr s)
 ///
 /// @param s the second version of the non-static data member.
 static bool
-non_static_data_member_type_size_changed(decl_base_sptr f, decl_base_sptr s)
+non_static_data_member_type_size_changed(const decl_base_sptr& f,
+					 const decl_base_sptr& s)
 {
   if (!is_member_decl(f)
       || !is_member_decl(s))
@@ -311,7 +319,8 @@ non_static_data_member_type_size_changed(decl_base_sptr f, decl_base_sptr s)
 ///
 /// @param s the second version of the static data member.
 static bool
-static_data_member_type_size_changed(decl_base_sptr f, decl_base_sptr s)
+static_data_member_type_size_changed(const decl_base_sptr& f,
+				     const decl_base_sptr& s)
 {
   if (!is_member_decl(f)
       || !is_member_decl(s))
@@ -336,7 +345,7 @@ static_data_member_type_size_changed(decl_base_sptr f, decl_base_sptr s)
 ///
 /// @return true if d1 and d2 are different but compatible.
 static bool
-is_compatible_change(decl_base_sptr d1, decl_base_sptr d2)
+is_compatible_change(const decl_base_sptr& d1, const decl_base_sptr& d2)
 {
   if ((d1 && d2)
       && (d1 != d2)
@@ -353,10 +362,18 @@ is_compatible_change(decl_base_sptr d1, decl_base_sptr d2)
 ///
 /// @return true if d1 and d2 have different names.
 static bool
-decl_name_changed(decl_base_sptr d1, decl_base_sptr d2)
+decl_name_changed(const type_or_decl_base* a1, const type_or_decl_base *a2)
 {
   string d1_name, d2_name;
 
+  const decl_base *d1 = dynamic_cast<const decl_base*>(a1);
+  if (d1 == 0)
+    return false;
+
+  const decl_base *d2 = dynamic_cast<const decl_base*>(a2);
+  if (d2 == 0)
+    return false;
+
   if (d1)
     d1_name = d1->get_qualified_name();
   if (d2)
@@ -365,6 +382,29 @@ decl_name_changed(decl_base_sptr d1, decl_base_sptr d2)
   return d1_name != d2_name;
 }
 
+/// Test if two decls have different names.
+///
+/// @param d1 the first declaration to consider.
+///
+/// @param d2 the second declaration to consider.
+///
+/// @return true if d1 and d2 have different names.
+static bool
+decl_name_changed(const type_or_decl_base_sptr& d1,
+		  const type_or_decl_base_sptr& d2)
+{return decl_name_changed(d1.get(), d2.get());}
+
+/// Test if a diff nodes carries a changes in which two decls have
+/// different names.
+///
+/// @param d the diff node to consider.
+///
+/// @return true iff d carries a changes in which two decls have
+/// different names.
+static bool
+decl_name_changed(const diff *d)
+{return decl_name_changed(d->first_subject(), d->second_subject());}
+
 /// Test if two decls represents a harmless name change.
 ///
 /// For now, a harmless name change is a name change for a typedef,
@@ -376,7 +416,7 @@ decl_name_changed(decl_base_sptr d1, decl_base_sptr d2)
 ///
 /// @return true iff decl @p s represents a harmless change over @p f.
 bool
-has_harmless_name_change(decl_base_sptr f, decl_base_sptr s)
+has_harmless_name_change(const decl_base_sptr& f, const decl_base_sptr& s)
 {
   return (decl_name_changed(f, s)
 	  && ((is_typedef(f) && is_typedef(s))
@@ -455,6 +495,52 @@ static_data_member_added_or_removed(const class_diff* diff)
   return false;
 }
 
+/// Test if a class_diff node has a harmless "One Definition Rule"
+/// violation that will cause a diagnostic rule.
+///
+/// The conditions this function looks for are:
+///
+///  1/ The two subject of the diff must be canonically different
+///
+///  2/ The two subjects of the diff must be structurally equal
+///
+///  3/ The canonical types of the subjects of the diff must be
+///  structurally different.
+///
+/// These conditions makes the diff node appears as it carries changes
+/// (because of a ODR glitch present in the binary), but the glitch
+/// has no effect on the structural equality of the subjects of the
+/// diff.  If we do not detect these conditions, we'd end up with a
+/// diagnostic glitch where the reporter thinks there is an ABI change
+/// (because of the canonical difference), but then it fails to give
+/// any detail about it, because there is no structural change.
+///
+/// @param diff the diff node to consider.
+///
+/// @return true iff the the diff node has a harmless "One Definition
+/// Rule" violation that cause an empty false positive.
+static bool
+class_diff_has_harmless_odr_violation_change(const diff* dif)
+{
+  class_diff* d =  dynamic_cast<class_diff*>(const_cast<diff*>(dif));
+  if (!d || !d->has_changes())
+    return false;
+
+  class_decl_sptr first = d->first_class_decl();
+  class_decl_sptr second = d->second_class_decl();
+
+  if (equals(*first, *second, 0))
+    {
+      class_decl_sptr fc = is_class_type(first->get_canonical_type());
+      class_decl_sptr sc = is_class_type(second->get_canonical_type());
+
+      if (!equals(*fc, *sc, 0))
+	return true;
+    }
+
+  return false;
+}
+
 /// Test if a class_diff node has static members added or
 /// removed.
 ///
@@ -532,7 +618,7 @@ has_virtual_mem_fn_change(const class_diff* diff)
        i != diff->changed_member_fns().end();
        ++i)
     if (get_member_function_is_virtual((*i)->first_function_decl())
-	|| !get_member_function_is_virtual((*i)->second_function_decl()))
+	|| get_member_function_is_virtual((*i)->second_function_decl()))
       {
 	if (get_member_function_vtable_offset((*i)->first_function_decl())
 	    == get_member_function_vtable_offset((*i)->second_function_decl()))
@@ -544,6 +630,44 @@ has_virtual_mem_fn_change(const class_diff* diff)
   return false;
 }
 
+/// Test if the function_decl_diff node has a change involving virtual
+/// member functions.
+///
+/// That means whether there is an added, removed or changed virtual
+/// member function.
+///
+/// @param diff the function_decl_diff node to consider.
+///
+/// @return true iff the function_decl_diff node contains changes
+/// involving virtual member functions.
+bool
+has_virtual_mem_fn_change(const function_decl_diff* diff)
+{
+  if (!diff)
+    return false;
+
+  function_decl_sptr ff = diff->first_function_decl(),
+    sf = diff->second_function_decl();
+
+  if (!is_member_function(ff)
+      || !is_member_function(sf))
+    return false;
+
+  bool ff_is_virtual = get_member_function_is_virtual(ff),
+    sf_is_virtual = get_member_function_is_virtual(sf);
+
+  if (ff_is_virtual != sf_is_virtual)
+    return true;
+
+  size_t ff_vtable_offset = get_member_function_vtable_offset(ff),
+    sf_vtable_offset = get_member_function_vtable_offset(sf);
+
+  if (ff_vtable_offset != sf_vtable_offset)
+    return true;
+
+  return false;
+}
+
 /// Test if the class_diff node has a change involving virtual member
 /// functions.
 ///
@@ -556,7 +680,10 @@ has_virtual_mem_fn_change(const class_diff* diff)
 /// virtual member functions.
 static bool
 has_virtual_mem_fn_change(const diff* diff)
-{return has_virtual_mem_fn_change(dynamic_cast<const class_diff*>(diff));}
+{
+  return (has_virtual_mem_fn_change(dynamic_cast<const class_diff*>(diff))
+	  || has_virtual_mem_fn_change(dynamic_cast<const function_decl_diff*>(diff)));
+}
 
 /// Test if the class_diff has changes to non virtual member
 /// functions.
@@ -629,6 +756,72 @@ static bool
 base_classes_added_or_removed(const diff* diff)
 {return base_classes_added_or_removed(dynamic_cast<const class_diff*>(diff));}
 
+/// Test if two @ref class_or_union_sptr are different just by the
+/// fact that one is decl-only and the other one is defined.
+///
+/// @param first the first class or union to consider.
+///
+/// @param second the second class or union to consider.
+///
+/// @return true iff the two arguments are different just by the fact
+/// that one is decl-only and the other one is defined.
+bool
+has_class_decl_only_def_change(const class_or_union_sptr& first,
+			       const class_or_union_sptr& second)
+{
+  if (!first || !second)
+    return false;
+
+  class_or_union_sptr f =
+    look_through_decl_only_class(first);
+  class_or_union_sptr s =
+    look_through_decl_only_class(second);
+
+  if (f->get_qualified_name() != s->get_qualified_name())
+    return false;
+
+  return (f->get_is_declaration_only() != s->get_is_declaration_only());
+}
+
+/// Test if a class_or_union_diff carries a change in which the two
+/// classes are different by the fact that one is a decl-only and the
+/// other one is defined.
+///
+/// @param diff the diff node to consider.
+////
+//// @return true if the class_or_union_diff carries a change in which
+/// the two classes are different by the fact that one is a decl-only
+/// and the other one is defined.
+static bool
+has_class_decl_only_def_change(const diff *diff)
+{
+  const class_or_union_diff *d = dynamic_cast<const class_or_union_diff*>(diff);
+  if (!d)
+    return false;
+
+  class_or_union_sptr f =
+    look_through_decl_only_class(d->first_class_or_union());
+  class_or_union_sptr s =
+    look_through_decl_only_class(d->second_class_or_union());
+
+  return has_class_decl_only_def_change(f, s);
+}
+
+/// Test if a diff node carries a basic type name change.
+///
+/// @param d the diff node to consider.
+///
+/// @return true iff the diff node carries a basic type name change.
+bool
+has_basic_type_name_change(const diff *d)
+{
+  if (const type_decl_diff *dif = is_diff_of_basic_type(d))
+    if (decl_name_changed(dif))
+      return true;
+
+  return false;
+}
+
 /// Test if an enum_diff carries an enumerator insertion.
 ///
 /// @param diff the enum_diff to consider.
@@ -670,8 +863,67 @@ has_harmful_enum_change(const diff* diff)
   return false;
 }
 
-/// The visiting code of the harmless_filter.
+/// Test if an @ref fn_parm_diff node has a top cv qualifier change on
+/// the type of the function parameter.
 ///
+/// @param diff the diff node to consider.  It should be a @ref
+/// fn_parm_diff, otherwise the function returns 'false' directly.
+///
+/// @return true iff @p diff is a @ref fn_parm_diff node that has a
+/// top cv qualifier change on the type of the function parameter.
+static bool
+has_fn_parm_type_cv_qual_change(const diff* diff)
+{
+  // is diff a "function parameter diff node?
+  const fn_parm_diff* parm_diff = is_fn_parm_diff(diff);
+
+  if (!parm_diff || !parm_diff->has_changes())
+    // diff either carries no change or is not a function parameter
+    // diff node.  So bail out.
+    return false;
+
+  function_decl::parameter_sptr first_parm = parm_diff->first_parameter();
+  function_decl::parameter_sptr second_parm = parm_diff->second_parameter();
+
+  type_base_sptr first_parm_type = first_parm->get_type();
+  type_base_sptr second_parm_type = second_parm->get_type();
+
+  if (!is_qualified_type(first_parm_type)
+      && !is_qualified_type(second_parm_type))
+    // None of the parameter types is qualified.
+    return false;
+
+  qualified_type_def::CV cv_quals_1 = qualified_type_def::CV_NONE;
+  qualified_type_def::CV cv_quals_2 = qualified_type_def::CV_NONE;
+  type_base_sptr peeled_type_1 = first_parm_type;
+  type_base_sptr peeled_type_2 = second_parm_type;
+
+  if (qualified_type_def_sptr qtype1 = is_qualified_type(first_parm_type))
+    {
+      cv_quals_1 = qtype1->get_cv_quals();
+      peeled_type_1 = peel_qualified_type(qtype1);
+    }
+
+  if (qualified_type_def_sptr qtype2 = is_qualified_type(second_parm_type))
+    {
+      cv_quals_2 = qtype2->get_cv_quals();
+      peeled_type_2 = peel_qualified_type(qtype2);
+    }
+
+  if (peeled_type_1
+      && peeled_type_2
+      && get_type_name(peeled_type_1) == get_type_name(peeled_type_2)
+      && cv_quals_1 != cv_quals_2)
+    // The top-level CV qualifiers of the function type are different
+    // and the un-qualified variant (peeled) of said function types
+    // are equal.  This means the only change the function types have
+    // are about top-level CV qualifiers.
+    return true;
+
+  return false;
+
+}
+
 /// Detect if the changes carried by a given diff node are deemed
 /// harmless and do categorize the diff node accordingly.
 ///
@@ -682,18 +934,21 @@ has_harmful_enum_change(const diff* diff)
 ///
 /// @return true iff the traversal shall keep going after the
 /// completion of this function.
-bool
-harmless_filter::visit(diff* d, bool pre)
+static bool
+categorize_harmless_diff_node(diff *d, bool pre)
 {
   if (!d->has_changes())
     return true;
 
-  diff_category category = NO_CHANGE_CATEGORY;
-
   if (pre)
     {
-      decl_base_sptr f = d->first_subject(),
-	s = d->second_subject();
+      diff_category category = NO_CHANGE_CATEGORY;
+
+      decl_base_sptr f = is_decl(d->first_subject()),
+	s = is_decl(d->second_subject());
+
+      if (has_class_decl_only_def_change(d))
+	category |= CLASS_DECL_ONLY_DEF_CHANGE_CATEGORY;
 
       if (access_changed(f, s))
 	category |= ACCESS_CHANGE_CATEGORY;
@@ -701,7 +956,8 @@ harmless_filter::visit(diff* d, bool pre)
       if (is_compatible_change(f, s))
 	category |= COMPATIBLE_TYPE_CHANGE_CATEGORY;
 
-      if (has_harmless_name_change(f, s))
+      if (has_harmless_name_change(f, s)
+	  || class_diff_has_harmless_odr_violation_change(d))
 	category |= HARMLESS_DECL_NAME_CHANGE_CATEGORY;
 
       if (has_non_virtual_mem_fn_change(d))
@@ -718,48 +974,72 @@ harmless_filter::visit(diff* d, bool pre)
       if (function_name_changed_but_not_symbol(d))
 	category |= HARMLESS_SYMBOL_ALIAS_CHANGE_CATEORY;
 
+      if (has_fn_parm_type_cv_qual_change(d))
+	category |= FN_PARM_TYPE_TOP_CV_CHANGE_CATEGORY;
+
       if (category)
 	{
-	  d->add_to_category(category);
+	  d->add_to_local_and_inherited_categories(category);
 	  // Also update the category of the canonical node.
 	  if (diff * canonical = d->get_canonical_diff())
-	    canonical->add_to_category(category);
+	    canonical->add_to_local_and_inherited_categories(category);
 	}
     }
 
   return true;
 }
 
-/// Part of the visiting code of the harmless_filter.
+/// Detect if the changes carried by a given diff node are deemed
+/// harmful and do categorize the diff node accordingly.
 ///
-/// This function is called after the visiting of a given diff node.
-/// Note that when this function is called, the visiting might not
-/// have taken place *if* the node (or an equivalent node) has already
-/// been visited.
+/// @param d the diff node being visited.
 ///
-/// @param d the diff node that has either been visited or skipped
-/// (because it has already been visited during this traversing).
-void
-harmless_filter::visit_end(diff* d)
+/// @param pre this is true iff the node is being visited *before* the
+/// children nodes of @p d.
+///
+/// @return true iff the traversal shall keep going after the
+/// completion of this function.
+static bool
+categorize_harmful_diff_node(diff *d, bool pre)
 {
-  if (d->context()->diff_has_been_visited(d))
+  if (!d->has_changes())
+    return true;
+
+  if (pre)
     {
-      // This node or one of its equivalent node has already been
-      // visited.  That means at this moment, harmless_filter::visit()
-      // has *not* been called prior to this
-      // harmless_filter::visit_end() is called.  In other words, only
-      // harmless_filter::visit_begin() and
-      // harmless_filter::visit_end() are called.
+      diff_category category = NO_CHANGE_CATEGORY;
+      decl_base_sptr f = is_decl(d->first_subject()),
+	s = is_decl(d->second_subject());
+
+      // Detect size or offset changes as well as data member addition
+      // or removal.
       //
-      // So let's update the category of this diff node from its
-      // canonical node.
-      diff* canonical = d->get_canonical_diff();
-      if (canonical)
-	d->add_to_category(canonical->get_category());
+      // TODO: be more specific -- not all size changes are harmful.
+      if (!has_class_decl_only_def_change(d)
+	  && (type_size_changed(f, s)
+	      || data_member_offset_changed(f, s)
+	      || non_static_data_member_type_size_changed(f, s)
+	      || non_static_data_member_added_or_removed(d)
+	      || base_classes_added_or_removed(d)
+	      || has_harmful_enum_change(d)))
+	category |= SIZE_OR_OFFSET_CHANGE_CATEGORY;
+
+      if (has_virtual_mem_fn_change(d))
+	category |= VIRTUAL_MEMBER_CHANGE_CATEGORY;
+
+      if (category)
+	{
+	  d->add_to_local_and_inherited_categories(category);
+	  // Update the category of the canonical diff node too.
+	  if (diff * canonical = d->get_canonical_diff())
+	    canonical->add_to_local_and_inherited_categories(category);
+	}
     }
+
+  return true;
 }
 
-/// The visiting code of the harmful_filter.
+/// The visiting code of the harmless_harmful_filter.
 ///
 /// @param d the diff node being visited.
 ///
@@ -769,46 +1049,13 @@ harmless_filter::visit_end(diff* d)
 /// @return true iff the traversal shall keep going after the
 /// completion of this function.
 bool
-harmful_filter::visit(diff* d, bool pre)
+harmless_harmful_filter::visit(diff* d, bool pre)
 {
-  diff_category category = NO_CHANGE_CATEGORY;
-
-  if (!d->has_changes())
-    return true;
-
-  if (pre)
-    {
-      decl_base_sptr f = d->first_subject(),
-	s = d->second_subject();
-
-      // Detect size or offset changes as well as data member addition
-      // or removal.
-      //
-      // TODO: be more specific -- not all size changes are harmful.
-      if (type_size_changed(f, s)
-	  || data_member_offset_changed(f, s)
-	  || non_static_data_member_type_size_changed(f, s)
-	  || non_static_data_member_added_or_removed(d)
-	  || base_classes_added_or_removed(d)
-	  || has_harmful_enum_change(d))
-	category |= SIZE_OR_OFFSET_CHANGE_CATEGORY;
-
-      if (has_virtual_mem_fn_change(d))
-	category |= VIRTUAL_MEMBER_CHANGE_CATEGORY;
-
-      if (category)
-	{
-	  d->add_to_category(category);
-	  // Update the category of the canonical diff node too.
-	  if (diff * canonical = d->get_canonical_diff())
-	    canonical->add_to_category(category);
-	}
-    }
-
-  return true;
+  return (categorize_harmless_diff_node(d, pre)
+	  && categorize_harmful_diff_node(d, pre));
 }
 
-/// Part of the visiting code of the harmless_filter.
+/// Part of the visiting code of the harmless_harmful_filter.
 ///
 /// This function is called after the visiting of a given diff node.
 /// Note that when this function is called, the visiting might not
@@ -818,25 +1065,23 @@ harmful_filter::visit(diff* d, bool pre)
 /// @param d the diff node that has either been visited or skipped
 /// (because it has already been visited during this traversing).
 void
-harmful_filter::visit_end(diff* d)
+harmless_harmful_filter::visit_end(diff* d)
 {
   if (d->context()->diff_has_been_visited(d))
     {
       // This node or one of its equivalent node has already been
-      // visited.  That means at this moment, harmful_filter::visit()
-      // has *not* been called prior to this
-      // harmful_filter::visit_end() is called.  In other words, only
-      // harmful_filter::visit_begin() and harmful_filter::visit_end()
-      // are called.
+      // visited.  That means at this moment,
+      // harmless_harmful_filter::visit() has *not* been called prior
+      // to this harmless_harmful_filter::visit_end() is called.  In
+      // other words, only harmless_harmful_filter::visit_begin() and
+      // harmless_harmful_filter::visit_end() are called.
       //
-      // So let's update the category of this diff node from it's
-      // cnanonical node.
-      diff* canonical = d->get_canonical_diff();
-      if (canonical)
-	d->add_to_category(canonical->get_category());
+      // So let's update the category of this diff node from its
+      // canonical node.
+      if (diff* c = d->get_canonical_diff())
+	d->add_to_local_and_inherited_categories(c->get_local_category());
     }
 }
-
 } // end namespace filtering
 } // end namespace comparison
 } // end namespace abigail
