@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2015 Red Hat, Inc.
+// Copyright (C) 2013-2016 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -20,34 +20,42 @@
 
 /// @file
 
-#include "config.h"
+#include "abg-internal.h"
+// <headers defining libabigail's API go under here>
+ABG_BEGIN_EXPORT_DECLARATIONS
+
 #include "abg-config.h"
 #include "abg-version.h"
 
+ABG_END_EXPORT_DECLARATIONS
+// </headers defining libabigail's API>
+
 namespace abigail
 {
 config::config()
-: m_format_minor(0), m_format_major(1),
-  m_xml_element_indent(2),
-  m_tu_instr_suffix(".bi"),
-  m_tu_instr_archive_suffix (".abi")
-{ }
+  : m_format_minor("0"),
+    m_format_major("1"),
+    m_xml_element_indent(2),
+    m_tu_instr_suffix(".bi"),
+    m_tu_instr_archive_suffix(".abi")
+{
+}
 
-unsigned char
+const std::string&
 config::get_format_minor_version_number() const
-{ return m_format_minor; }
+{return m_format_minor;}
 
 void
-config::set_format_minor_version_number(unsigned char v)
-{ m_format_minor = v; }
+config::set_format_minor_version_number(const std::string& v)
+{m_format_minor = v;}
 
-unsigned char
+const std::string&
 config::get_format_major_version_number() const
-{ return m_format_major; }
+{return m_format_major;}
 
 void
-config::set_format_major_version_number(unsigned char v)
-{ m_format_major= v; }
+config::set_format_major_version_number(const std::string& v)
+{m_format_major= v;}
 
 unsigned
 config::get_xml_element_indent() const
@@ -55,28 +63,30 @@ config::get_xml_element_indent() const
 
 void
 config::set_xml_element_indent(unsigned indent)
-{ m_xml_element_indent = indent; }
+{m_xml_element_indent = indent;}
 
 const std::string&
 config::get_tu_instr_suffix() const
-{ return m_tu_instr_suffix; }
+{return m_tu_instr_suffix;}
 
 void
 config::set_tu_instr_suffix(const std::string& s)
-{ m_tu_instr_suffix = s; }
+{m_tu_instr_suffix = s;}
 
 const std::string&
 config::get_tu_instr_archive_suffix() const
-{ return m_tu_instr_archive_suffix; }
+{return m_tu_instr_archive_suffix;}
 
 void
 config::set_tu_instr_archive_suffix(const std::string& s)
-{ m_tu_instr_archive_suffix = s; }
+{m_tu_instr_archive_suffix = s; }
 
 extern "C"
 {
 void
-abigail_get_library_version(int& major, int& minor, int& revision)
+abigail_get_library_version(std::string& major,
+			    std::string& minor,
+			    std::string& revision)
 {
   major = ABIGAIL_VERSION_MAJOR;
   minor = ABIGAIL_VERSION_MINOR;
