@@ -1,6 +1,6 @@
 // -*- mode: C++ -*-
 //
-// Copyright (C) 2013-2015 Red Hat, Inc.
+// Copyright (C) 2013-2016 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -20,9 +20,16 @@
 
 /// @file
 
+#include "abg-internal.h"
+// <headers defining libabigail's API go under here>
+ABG_BEGIN_EXPORT_DECLARATIONS
+
 #include "abg-hash.h"
 #include "abg-ir.h"
 
+ABG_END_EXPORT_DECLARATIONS
+// </headers defining libabigail's API>
+
 namespace abigail
 {
 
@@ -104,23 +111,19 @@ struct decl_base::hash
   size_t
   operator()(const decl_base& d) const
   {
-    if (d.peek_hash_value() == 0)
-      {
-	std::tr1::hash<string> str_hash;
+    std::tr1::hash<string> str_hash;
 
-	size_t v = str_hash(typeid(d).name());
-	if (!d.get_linkage_name().empty())
-	  v = hashing::combine_hashes(v, str_hash(d.get_linkage_name()));
-	else if (!d.get_name().empty())
-	  v = hashing::combine_hashes(v, str_hash(d.get_qualified_name()));
-	if (is_member_decl(d))
-	  {
-	    v = hashing::combine_hashes(v, get_member_access_specifier(d));
-	    v = hashing::combine_hashes(v, get_member_is_static(d));
-	  }
-	d.set_hash(v);
+    size_t v = str_hash(typeid(d).name());
+    if (!d.get_linkage_name().empty())
+      v = hashing::combine_hashes(v, str_hash(d.get_linkage_name()));
+    if (!d.get_name().empty())
+      v = hashing::combine_hashes(v, str_hash(d.get_qualified_name()));
+    if (is_member_decl(d))
+      {
+	v = hashing::combine_hashes(v, get_member_access_specifier(d));
+	v = hashing::combine_hashes(v, get_member_is_static(d));
       }
-    return d.peek_hash_value();
+    return v;
   }
 }; // end struct decl_base::hash
 
@@ -129,18 +132,15 @@ struct type_decl::hash
   size_t
   operator()(const type_decl& t) const
   {
-    if (t.peek_hash_value() == 0 || t.hashing_started())
-      {
-	decl_base::hash decl_hash;
-	type_base::hash type_hash;
-	std::tr1::hash<string> str_hash;
+    decl_base::hash decl_hash;
+    type_base::hash type_hash;
+    std::tr1::hash<string> str_hash;
 
-	size_t v = str_hash(typeid(t).name());
-	v = hashing::combine_hashes(v, decl_hash(t));
-	v = hashing::combine_hashes(v, type_hash(t));
-	t.set_hash(v);
-      }
-    return t.peek_hash_value();
+    size_t v = str_hash(typeid(t).name());
+    v = hashing::combine_hashes(v, decl_hash(t));
+    v = hashing::combine_hashes(v, type_hash(t));
+
+    return v;
   }
 };
 
@@ -152,18 +152,15 @@ struct type_decl::hash
 size_t
 scope_decl::hash::operator()(const scope_decl& d) const
 {
-  if (d.peek_hash_value() == 0 || d.hashing_started())
-    {
-      std::tr1::hash<string> hash_string;
-      size_t v = hash_string(typeid(d).name());
-      for (scope_decl::declarations::const_iterator i =
-	     d.get_member_decls().begin();
-	   i != d.get_member_decls().end();
-	   ++i)
-	v = hashing::combine_hashes(v, (*i)->get_hash());
-      d.set_hash(v);
-    }
-  return d.peek_hash_value();
+  std::tr1::hash<string> hash_string;
+  size_t v = hash_string(typeid(d).name());
+  for (scope_decl::declarations::const_iterator i =
+	 d.get_member_decls().begin();
+       i != d.get_member_decls().end();
+       ++i)
+    v = hashing::combine_hashes(v, (*i)->get_hash());
+
+  return v;
 }
 
 /// Hashing operator for the @ref scope_decl type.
@@ -180,19 +177,15 @@ struct scope_type_decl::hash
   size_t
   operator()(const scope_type_decl& t) const
   {
-    if (t.peek_hash_value() == 0 || t.hashing_started())
-      {
-	decl_base::hash decl_hash;
-	type_base::hash type_hash;
-	std::tr1::hash<string> str_hash;
+    decl_base::hash decl_hash;
+    type_base::hash type_hash;
+    std::tr1::hash<string> str_hash;
 
-	size_t v = str_hash(typeid(t).name());
-	v = hashing::combine_hashes(v, decl_hash(t));
-	v = hashing::combine_hashes(v, type_hash(t));
-	t.set_hash(v);
-      }
+    size_t v = str_hash(typeid(t).name());
+    v = hashing::combine_hashes(v, decl_hash(t));
+    v = hashing::combine_hashes(v, type_hash(t));
 
-    return t.peek_hash_value();
+    return v;
   }
 };
 
@@ -201,19 +194,15 @@ struct qualified_type_def::hash
   size_t
   operator()(const qualified_type_def& t) const
   {
-    if (t.peek_hash_value() == 0 || t.hashing_started())
-      {
-	type_base::hash type_hash;
-	decl_base::hash decl_hash;
-	std::tr1::hash<string> str_hash;
+    type_base::hash type_hash;
+    decl_base::hash decl_hash;
+    std::tr1::hash<string> str_hash;
 
-	size_t v = str_hash(typeid(t).name());
-	v = hashing::combine_hashes(v, type_hash(t));
-	v = hashing::combine_hashes(v, decl_hash(t));
-	v = hashing::combine_hashes(v, t.get_cv_quals());
-	t.set_hash(v);
-      }
-    return t.peek_hash_value();
+    size_t v = str_hash(typeid(t).name());
+    v = hashing::combine_hashes(v, type_hash(t));
+    v = hashing::combine_hashes(v, decl_hash(t));
+    v = hashing::combine_hashes(v, t.get_cv_quals());
+    return v;
   }
 };
 
@@ -222,21 +211,16 @@ struct pointer_type_def::hash
   size_t
   operator()(const pointer_type_def& t) const
   {
-    if (t.peek_hash_value() == 0 || t.hashing_started())
-      {
-	std::tr1::hash<string> str_hash;
-	type_base::hash type_base_hash;
-	decl_base::hash decl_hash;
-	type_base::shared_ptr_hash hash_type_ptr;
+    std::tr1::hash<string> str_hash;
+    type_base::hash type_base_hash;
+    decl_base::hash decl_hash;
+    type_base::shared_ptr_hash hash_type_ptr;
 
-	size_t v = str_hash(typeid(t).name());
-	v = hashing::combine_hashes(v, decl_hash(t));
-	v = hashing::combine_hashes(v, type_base_hash(t));
-	v = hashing::combine_hashes(v, hash_type_ptr(t.get_pointed_to_type()));
-	t.set_hash(v);
-      }
-
-    return t.peek_hash_value();
+    size_t v = str_hash(typeid(t).name());
+    v = hashing::combine_hashes(v, decl_hash(t));
+    v = hashing::combine_hashes(v, type_base_hash(t));
+    v = hashing::combine_hashes(v, hash_type_ptr(t.get_pointed_to_type()));
+    return v ;
   }
 };
 
@@ -245,24 +229,19 @@ struct reference_type_def::hash
   size_t
   operator()(const reference_type_def& t)
   {
-    if (t.peek_hash_value() == 0 || t.hashing_started())
-      {
-	std::tr1::hash<string> hash_str;
-	type_base::hash hash_type_base;
-	decl_base::hash hash_decl;
-	type_base::shared_ptr_hash hash_type_ptr;
+    std::tr1::hash<string> hash_str;
+    type_base::hash hash_type_base;
+    decl_base::hash hash_decl;
+    type_base::shared_ptr_hash hash_type_ptr;
 
-	size_t v = hash_str(typeid(t).name());
-	v = hashing::combine_hashes(v, hash_str(t.is_lvalue()
-						? "lvalue"
-						: "rvalue"));
-	v = hashing::combine_hashes(v, hash_type_base(t));
-	v = hashing::combine_hashes(v, hash_decl(t));
-	v = hashing::combine_hashes(v, hash_type_ptr(t.get_pointed_to_type()));
-	t.set_hash(v);
-      }
-
-    return t.peek_hash_value();
+    size_t v = hash_str(typeid(t).name());
+    v = hashing::combine_hashes(v, hash_str(t.is_lvalue()
+					    ? "lvalue"
+					    : "rvalue"));
+    v = hashing::combine_hashes(v, hash_type_base(t));
+    v = hashing::combine_hashes(v, hash_decl(t));
+    v = hashing::combine_hashes(v, hash_type_ptr(t.get_pointed_to_type()));
+    return v;
   }
 };
 
@@ -283,29 +262,25 @@ struct array_type_def::hash
   size_t
   operator()(const array_type_def& t)
   {
-    if (t.peek_hash_value() == 0 || t.hashing_started())
-      {
-	std::tr1::hash<string> hash_str;
-	type_base::hash hash_type_base;
-	decl_base::hash hash_decl;
-	type_base::shared_ptr_hash hash_type_ptr;
-	array_type_def::subrange_type::hash hash_subrange;
+    std::tr1::hash<string> hash_str;
+    type_base::hash hash_type_base;
+    decl_base::hash hash_decl;
+    type_base::shared_ptr_hash hash_type_ptr;
+    array_type_def::subrange_type::hash hash_subrange;
 
-	size_t v = hash_str(typeid(t).name());
+    size_t v = hash_str(typeid(t).name());
 
-	v = hashing::combine_hashes(v, hash_type_base(t));
-	v = hashing::combine_hashes(v, hash_decl(t));
-	v = hashing::combine_hashes(v, hash_type_ptr(t.get_element_type()));
+    v = hashing::combine_hashes(v, hash_type_base(t));
+    v = hashing::combine_hashes(v, hash_decl(t));
+    v = hashing::combine_hashes(v, hash_type_ptr(t.get_element_type()));
 
-	for (vector<array_type_def::subrange_sptr >::const_iterator i =
-	     t.get_subranges().begin();
-	     i != t.get_subranges().end();
-	     ++i)
-	  v = hashing::combine_hashes(v, hash_subrange(**i));
+    for (vector<array_type_def::subrange_sptr >::const_iterator i =
+	   t.get_subranges().begin();
+	 i != t.get_subranges().end();
+	 ++i)
+      v = hashing::combine_hashes(v, hash_subrange(**i));
 
-	t.set_hash(v);
-      }
-    return t.peek_hash_value();
+    return v;
   }
 };
 
@@ -314,27 +289,23 @@ struct enum_type_decl::hash
   size_t
   operator()(const enum_type_decl& t) const
   {
-    if (t.peek_hash_value() == 0 || t.hashing_started())
-      {
-	std::tr1::hash<string> str_hash;
-	decl_base::hash decl_hash;
-	type_base::shared_ptr_hash type_ptr_hash;
-	std::tr1::hash<size_t> size_t_hash;
+    std::tr1::hash<string> str_hash;
+    decl_base::hash decl_hash;
+    type_base::shared_ptr_hash type_ptr_hash;
+    std::tr1::hash<size_t> size_t_hash;
 
-	size_t v = str_hash(typeid(t).name());
-	v = hashing::combine_hashes(v, decl_hash(t));
-	v = hashing::combine_hashes(v, type_ptr_hash(t.get_underlying_type()));
-	for (enum_type_decl::enumerators::const_iterator i =
-	       t.get_enumerators().begin();
-	     i != t.get_enumerators().end();
-	     ++i)
-	  {
-	    v = hashing::combine_hashes(v, str_hash(i->get_name()));
-	    v = hashing::combine_hashes(v, size_t_hash(i->get_value()));
-	  }
-	t.set_hash(v);
+    size_t v = str_hash(typeid(t).name());
+    v = hashing::combine_hashes(v, decl_hash(t));
+    v = hashing::combine_hashes(v, type_ptr_hash(t.get_underlying_type()));
+    for (enum_type_decl::enumerators::const_iterator i =
+	   t.get_enumerators().begin();
+	 i != t.get_enumerators().end();
+	 ++i)
+      {
+	v = hashing::combine_hashes(v, str_hash(i->get_name()));
+	v = hashing::combine_hashes(v, size_t_hash(i->get_value()));
       }
-    return t.peek_hash_value();
+    return v;
   }
 };
 
@@ -343,22 +314,17 @@ struct typedef_decl::hash
   size_t
   operator()(const typedef_decl& t) const
   {
-    if (t.peek_hash_value() == 0 || t.hashing_started())
-      {
-	std::tr1::hash<string> str_hash;
-	type_base::hash hash_type;
-	decl_base::hash decl_hash;
-	type_base::shared_ptr_hash type_ptr_hash;
+    std::tr1::hash<string> str_hash;
+    type_base::hash hash_type;
+    decl_base::hash decl_hash;
+    type_base::shared_ptr_hash type_ptr_hash;
 
-	size_t v = str_hash(typeid(t).name());
-	v = hashing::combine_hashes(v, hash_type(t));
-	v = hashing::combine_hashes(v, decl_hash(t));
-	v = hashing::combine_hashes(v, type_ptr_hash(t.get_underlying_type()));
-	t.set_hash(v);
-      }
-
-    return t.peek_hash_value();
-   }
+    size_t v = str_hash(typeid(t).name());
+    v = hashing::combine_hashes(v, hash_type(t));
+    v = hashing::combine_hashes(v, decl_hash(t));
+    v = hashing::combine_hashes(v, type_ptr_hash(t.get_underlying_type()));
+    return v;
+  }
  };
 
 /// Compute a hash for an instance @ref var_decl.
@@ -374,24 +340,22 @@ struct typedef_decl::hash
 size_t
 var_decl::hash::operator()(const var_decl& t) const
 {
-  if (t.peek_hash_value() == 0 || t.hashing_started())
+  std::tr1::hash<string> hash_string;
+  decl_base::hash hash_decl;
+  type_base::shared_ptr_hash hash_type_ptr;
+  std::tr1::hash<size_t> hash_size_t;
+
+  size_t v = hash_string(typeid(t).name());
+  v = hashing::combine_hashes(v, hash_decl(t));
+  v = hashing::combine_hashes(v, hash_type_ptr(t.get_type()));
+
+  if (is_data_member(t) && get_data_member_is_laid_out(t))
     {
-      std::tr1::hash<string> hash_string;
-      decl_base::hash hash_decl;
-      type_base::shared_ptr_hash hash_type_ptr;
-      std::tr1::hash<size_t> hash_size_t;
-
-      size_t v = hash_string(typeid(t).name());
-      v = hashing::combine_hashes(v, hash_decl(t));
-      v = hashing::combine_hashes(v, hash_type_ptr(t.get_type()));
-
-      if (is_data_member(t) && get_data_member_is_laid_out(t))
-	v = hashing::combine_hashes(v,
-				    hash_size_t(get_data_member_offset(t)));
-
-      t.set_hash(v);
+      v = hashing::combine_hashes(v, hash_decl(*t.get_scope()));
+      v = hashing::combine_hashes(v, hash_size_t(get_data_member_offset(t)));
     }
-  return t.peek_hash_value();
+
+  return v;
 }
 
 /// Compute a hash for a pointer to @ref var_decl.
@@ -415,39 +379,37 @@ var_decl::hash::operator()(const var_decl* t) const
 size_t
 function_decl::hash::operator()(const function_decl& t) const
 {
-  if (t.peek_hash_value() == 0 || t.hashing_started())
+  std::tr1::hash<int> hash_int;
+  std::tr1::hash<size_t> hash_size_t;
+  std::tr1::hash<bool> hash_bool;
+  std::tr1::hash<string> hash_string;
+  decl_base::hash hash_decl_base;
+  type_base::shared_ptr_hash hash_type_ptr;
+
+  size_t v = hash_string(typeid(t).name());
+  v = hashing::combine_hashes(v, hash_decl_base(t));
+  v = hashing::combine_hashes(v, hash_type_ptr(t.get_type()));
+  v = hashing::combine_hashes(v, hash_bool(t.is_declared_inline()));
+  v = hashing::combine_hashes(v, hash_int(t.get_binding()));
+  if (is_member_function(t))
     {
-      std::tr1::hash<int> hash_int;
-      std::tr1::hash<size_t> hash_size_t;
-      std::tr1::hash<bool> hash_bool;
-      std::tr1::hash<string> hash_string;
-      decl_base::hash hash_decl_base;
-      type_base::shared_ptr_hash hash_type_ptr;
+      bool is_ctor = get_member_function_is_ctor(t),
+	is_dtor = get_member_function_is_dtor(t),
+	is_static = get_member_is_static(t),
+	is_const = get_member_function_is_const(t);
+      size_t voffset = get_member_function_vtable_offset(t);
 
-      size_t v = hash_string(typeid(t).name());
-      v = hashing::combine_hashes(v, hash_decl_base(t));
-      v = hashing::combine_hashes(v, hash_type_ptr(t.get_type()));
-      v = hashing::combine_hashes(v, hash_bool(t.is_declared_inline()));
-      v = hashing::combine_hashes(v, hash_int(t.get_binding()));
-      if (is_member_function(t))
-	{
-	  bool is_ctor = get_member_function_is_ctor(t),
-	    is_dtor = get_member_function_is_dtor(t),
-	    is_static = get_member_is_static(t),
-	    is_const = get_member_function_is_const(t);
-	  size_t voffset = get_member_function_vtable_offset(t);
-
-	  v = hashing::combine_hashes(v, hash_bool(is_ctor));
-	  v = hashing::combine_hashes(v, hash_bool(is_dtor));
-	  v = hashing::combine_hashes(v, hash_bool(is_static));
-	  v = hashing::combine_hashes(v, hash_bool(is_const));
-	  if (!is_static && !is_ctor)
-	    v = hashing::combine_hashes(v, hash_size_t(voffset));
-	}
-      t.set_hash(v);
+      v = hashing::combine_hashes(v, hash_bool(is_ctor));
+      v = hashing::combine_hashes(v, hash_bool(is_dtor));
+      v = hashing::combine_hashes(v, hash_bool(is_static));
+      v = hashing::combine_hashes(v, hash_bool(is_const));
+      if (!is_static && !is_ctor)
+	v = hashing::combine_hashes(v, hash_size_t(voffset));
     }
-  return t.peek_hash_value();
+
+  return v;
 }
+
 /// Compute a hash for a pointer to @ref function_decl.
 ///
 /// @param t the pointer to @ref function_decl to compute the hash for.
@@ -564,7 +526,7 @@ function_type::hash::operator()(const function_type_sptr t) const
 // </struct function_type::hash stuff>
 
 size_t
-class_decl::member_base::hash::operator()(const member_base& m) const
+member_base::hash::operator()(const member_base& m) const
 {
   std::tr1::hash<int> hash_int;
   return hash_int(m.get_access_specifier());
@@ -588,7 +550,7 @@ class_decl::base_spec::hash::operator()(const base_spec& t) const
 }
 
 size_t
-class_decl::member_function_template::hash::operator()
+member_function_template::hash::operator()
   (const member_function_template& t) const
 {
   std::tr1::hash<bool> hash_bool;
@@ -606,7 +568,7 @@ class_decl::member_function_template::hash::operator()
 }
 
 size_t
-class_decl::member_class_template::hash::operator()
+member_class_template::hash::operator()
   (const member_class_template& t) const
 {
   member_base::hash hash_member;
@@ -620,6 +582,84 @@ class_decl::member_class_template::hash::operator()
   return v;
 }
 
+/// Compute a hash for a @ref class_or_union
+///
+/// @param t the class_or_union for which to compute the hash value.
+///
+/// @return the computed hash value.
+size_t
+class_or_union::hash::operator()(const class_or_union& t) const
+{
+  if (t.hashing_started()
+      || (t.get_is_declaration_only() && !t.get_definition_of_declaration()))
+    // All non-resolved decl-only types have a hash of zero.  Their hash
+    // will differ from the resolved hash, but then at least, having
+    // it be zero will give a hint that we couldn't actually compute
+    // the hash.
+    return 0;
+
+  // If the type is decl-only and now has a definition, then hash its
+  // definition instead.
+
+  if (t.get_is_declaration_only())
+    {
+      assert(t.get_definition_of_declaration());
+      size_t v = operator()(*t.get_definition_of_declaration());
+      return v;
+    }
+
+  assert(!t.get_is_declaration_only());
+
+  std::tr1::hash<string> hash_string;
+  scope_type_decl::hash hash_scope_type;
+  var_decl::hash hash_data_member;
+  member_function_template::hash hash_member_fn_tmpl;
+  member_class_template::hash hash_member_class_tmpl;
+
+  size_t v = hash_string(typeid(t).name());
+  v = hashing::combine_hashes(v, hash_scope_type(t));
+
+  t.hashing_started(true);
+
+  // Hash data members.
+  for (class_decl::data_members::const_iterator d =
+	 t.get_non_static_data_members().begin();
+       d != t.get_non_static_data_members().end();
+       ++d)
+    v = hashing::combine_hashes(v, hash_data_member(**d));
+
+  // Do not hash member functions. All of them are not necessarily
+  // emitted per class, in a given TU so do not consider them when
+  // hashing a class.
+
+  // Hash member function templates
+  for (member_function_templates::const_iterator f =
+	 t.get_member_function_templates().begin();
+       f != t.get_member_function_templates().end();
+       ++f)
+    v = hashing::combine_hashes(v, hash_member_fn_tmpl(**f));
+
+  // Hash member class templates
+  for (member_class_templates::const_iterator c =
+	 t.get_member_class_templates().begin();
+       c != t.get_member_class_templates().end();
+       ++c)
+    v = hashing::combine_hashes(v, hash_member_class_tmpl(**c));
+
+  t.hashing_started(false);
+
+  return v;
+};
+
+/// Compute a hash for a @ref class_or_union
+///
+/// @param t the class_or_union for which to compute the hash value.
+///
+/// @return the computed hash value.
+size_t
+class_or_union::hash::operator()(const class_or_union *t) const
+{return t ? operator()(*t) : 0;}
+
 /// Compute a hash for a @ref class_decl
 ///
 /// @param t the class_decl for which to compute the hash value.
@@ -628,79 +668,50 @@ class_decl::member_class_template::hash::operator()
 size_t
 class_decl::hash::operator()(const class_decl& t) const
 {
-    if (t.hashing_started())
-      return 0;
+  if (t.hashing_started()
+      || (t.get_is_declaration_only() && !t.get_definition_of_declaration()))
+    // All non-resolved decl-only types have a hash of zero.  Their hash
+    // will differ from the resolved hash, but then at least, having
+    // it be zero will give a hint that we couldn't actually compute
+    // the hash.
+    return 0;
 
-    if (t.peek_hash_value() == 0)
-      {
-	std::tr1::hash<string> hash_string;
-	std::tr1::hash<bool> hash_bool;
-#if 0
-	type_base::dynamic_hash hash_type;
-#endif
-	scope_type_decl::hash hash_scope_type;
-	class_decl::base_spec::hash hash_base;
-	var_decl::hash hash_data_member;
-	function_decl::hash hash_member_fn;
-	class_decl::member_function_template::hash hash_member_fn_tmpl;
-	class_decl::member_class_template::hash hash_member_class_tmpl;
 
-	size_t v = hash_string(typeid(t).name());
-	v = hashing::combine_hashes(v, hash_scope_type(t));
-	v = hashing::combine_hashes(v, hash_bool(t.get_is_declaration_only()));
+  // If the type is decl-only and now has a definition, then hash its
+  // definition instead.
 
-	t.hashing_started(true);
+  if (t.get_is_declaration_only())
+    {
+      assert(t.get_definition_of_declaration());
+      size_t v = operator()(*is_class_type(t.get_definition_of_declaration()));
+      return v;
+    }
 
-	// Hash bases.
-	for (class_decl::base_specs::const_iterator b =
-	       t.get_base_specifiers().begin();
-	     b != t.get_base_specifiers().end();
-	     ++b)
-	  v = hashing::combine_hashes(v, hash_base(**b));
+  assert(!t.get_is_declaration_only());
 
-	// Hash member types.
-#if 0
-	for (class_decl::member_types::const_iterator ti =
-	       t.get_member_types().begin();
-	     ti != t.get_member_types().end();
-	     ++ti)
-	  v = hashing::combine_hashes(v, hash_type((*ti).get()));
-#endif
+  std::tr1::hash<string> hash_string;
+  class_decl::base_spec::hash hash_base;
+  class_or_union::hash hash_class_or_union;
 
-	// Hash data members.
-	for (class_decl::data_members::const_iterator d =
-	       t.get_data_members().begin();
-	     d != t.get_data_members().end();
-	     ++d)
-	  v = hashing::combine_hashes(v, hash_data_member(**d));
+  size_t v = hash_string(typeid(t).name());
 
-	// Hash member_function
-	for (class_decl::member_functions::const_iterator f =
-	       t.get_member_functions().begin();
-	     f != t.get_member_functions().end();
-	     ++f)
-	  if (get_member_function_is_virtual(*f))
-	    v = hashing::combine_hashes(v, hash_member_fn(**f));
+  t.hashing_started(true);
 
-	// Hash member function templates
-	for (class_decl::member_function_templates::const_iterator f =
-	       t.get_member_function_templates().begin();
-	     f != t.get_member_function_templates().end();
-	     ++f)
-	  v = hashing::combine_hashes(v, hash_member_fn_tmpl(**f));
+  // Hash bases.
+  for (class_decl::base_specs::const_iterator b =
+	 t.get_base_specifiers().begin();
+       b != t.get_base_specifiers().end();
+       ++b)
+    {
+      class_decl_sptr cl = (*b)->get_base_class();
+      v = hashing::combine_hashes(v, hash_base(**b));
+    }
 
-	// Hash member class templates
-	for (class_decl::member_class_templates::const_iterator c =
-	       t.get_member_class_templates().begin();
-	     c != t.get_member_class_templates().end();
-	     ++c)
-	  v = hashing::combine_hashes(v, hash_member_class_tmpl(**c));
+  v = hashing::combine_hashes(v, hash_class_or_union(t));
 
-	t.hashing_started(false);
+  t.hashing_started(false);
 
-	t.set_hash(v);
-      }
-    return t.peek_hash_value();
+  return v;
 }
 
 /// Compute a hash for a @ref class_decl
@@ -762,7 +773,7 @@ template_decl::hash::operator()(const template_decl& t) const
   template_parameter::shared_ptr_hash hash_template_parameter;
 
   size_t v = hash_string(typeid(t).name());
-  v = hash_string(t.get_qualified_name());
+  v = hashing::combine_hashes(v, hash_string(t.get_qualified_name()));
 
   for (list<template_parameter_sptr>::const_iterator p =
 	 t.get_template_parameters().begin();
@@ -779,19 +790,15 @@ struct type_tparameter::hash
   size_t
   operator()(const type_tparameter& t) const
   {
-    if (t.peek_hash_value() == 0 || t.hashing_started())
-      {
-	std::tr1::hash<string> hash_string;
-	template_parameter::hash hash_template_parameter;
-	type_decl::hash hash_type;
+    std::tr1::hash<string> hash_string;
+    template_parameter::hash hash_template_parameter;
+    type_decl::hash hash_type;
 
-	size_t v = hash_string(typeid(t).name());
-	v = hashing::combine_hashes(v, hash_template_parameter(t));
-	v = hashing::combine_hashes(v, hash_type(t));
+    size_t v = hash_string(typeid(t).name());
+    v = hashing::combine_hashes(v, hash_template_parameter(t));
+    v = hashing::combine_hashes(v, hash_type(t));
 
-	t.set_hash(v);
-      }
-    return t.peek_hash_value();
+    return v;
   }
 };
 
@@ -829,19 +836,15 @@ struct template_tparameter::hash
   size_t
   operator()(const template_tparameter& t) const
   {
-    if (t.peek_hash_value() == 0 || t.hashing_started())
-      {
-	std::tr1::hash<string> hash_string;
-	type_tparameter::hash hash_template_type_parm;
-	template_decl::hash hash_template_decl;
+    std::tr1::hash<string> hash_string;
+    type_tparameter::hash hash_template_type_parm;
+    template_decl::hash hash_template_decl;
 
-	size_t v = hash_string(typeid(t).name());
-	v = hashing::combine_hashes(v, hash_template_type_parm(t));
-	v = hashing::combine_hashes(v, hash_template_decl(t));
-	t.set_hash(v);
-      }
+    size_t v = hash_string(typeid(t).name());
+    v = hashing::combine_hashes(v, hash_template_type_parm(t));
+    v = hashing::combine_hashes(v, hash_template_decl(t));
 
-    return t.peek_hash_value();
+    return v;
   }
 };
 
@@ -964,12 +967,13 @@ type_base::dynamic_hash::operator()(const type_base* t) const
 {
   if (t == 0)
     return 0;
-  if (const class_decl::member_function_template* d =
-      dynamic_cast<const class_decl::member_function_template*>(t))
-    return class_decl::member_function_template::hash()(*d);
-  if (const class_decl::member_class_template* d =
-      dynamic_cast<const class_decl::member_class_template*>(t))
-    return class_decl::member_class_template::hash()(*d);
+
+  if (const member_function_template* d =
+      dynamic_cast<const member_function_template*>(t))
+    return member_function_template::hash()(*d);
+  if (const member_class_template* d =
+      dynamic_cast<const member_class_template*>(t))
+    return member_class_template::hash()(*d);
   if (const template_tparameter* d =
       dynamic_cast<const template_tparameter*>(t))
     return template_tparameter::hash()(*d);
@@ -992,6 +996,8 @@ type_base::dynamic_hash::operator()(const type_base* t) const
     return typedef_decl::hash()(*d);
   if (const class_decl* d = dynamic_cast<const class_decl*>(t))
     return class_decl::hash()(*d);
+  if (const union_decl* d = dynamic_cast<const union_decl*>(t))
+    return union_decl::hash()(*d);
   if (const scope_type_decl* d = dynamic_cast<const scope_type_decl*>(t))
     return scope_type_decl::hash()(*d);
   if (const method_type* d = dynamic_cast<const method_type*>(t))
@@ -1007,14 +1013,4 @@ size_t
 type_base::shared_ptr_hash::operator()(const shared_ptr<type_base> t) const
 {return type_base::dynamic_hash()(t.get());}
 
-size_t
-type_base::cached_hash::operator()(const type_base* t) const
-{
-  const decl_base* d = get_type_declaration(t);
-  return d->get_hash();
-}
-
-size_t
-type_base::cached_hash::operator() (const type_base_sptr t) const
-{return type_base::cached_hash()(t.get());}
 }//end namespace abigail
