@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2015 Red Hat, Inc.
+// Copyright (C) 2013-2017 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -90,10 +90,10 @@ static InOutSpec specs[] =
     "output/test-abidiff/test-var0-report.txt"
   },
   {
-    "data/test-abidiff/test-corpus0-v0.so.abi",
-    "data/test-abidiff/test-corpus0-v1.so.abi",
-    "data/test-abidiff/test-corpus0-report0.txt",
-    "output/test-abidiff/test-corpus0-report0.txt"
+    "data/test-abidiff/test-PR18166-libtirpc.so.abi",
+    "data/test-abidiff/test-PR18166-libtirpc.so.abi",
+    "data/test-abidiff/test-PR18166-libtirpc.so.report.txt",
+    "output/test-abidiff/test-PR18166-libtirpc.so.report.txt"
   },
   {
     "data/test-abidiff/test-PR18791-v0.so.abi",
@@ -124,6 +124,8 @@ using abigail::xml_reader::read_corpus_from_native_xml_file;
 using abigail::comparison::corpus_diff_sptr;
 using abigail::comparison::translation_unit_diff_sptr;
 using abigail::comparison::compute_diff;
+using abigail::comparison::diff_context_sptr;
+using abigail::comparison::diff_context;
 
 int
 main(int, char*[])
@@ -131,7 +133,7 @@ main(int, char*[])
   bool is_ok = true;
 
   string out_path =
-    abigail::tests::get_build_dir() + "/tests/" + specs->out_path;
+    string(abigail::tests::get_build_dir()) + "/tests/" + specs->out_path;
   if (!abigail::tools_utils::ensure_parent_dir_created(out_path))
     {
       cerr << "Could not create parent director for " << out_path;
@@ -142,13 +144,13 @@ main(int, char*[])
   for (InOutSpec *s = specs; s->first_in_path; ++s)
     {
       first_in_path =
-	abigail::tests::get_src_dir() + "/tests/" + s->first_in_path;
+	string(abigail::tests::get_src_dir()) + "/tests/" + s->first_in_path;
       second_in_path =
-	abigail::tests::get_src_dir() + "/tests/" + s->second_in_path;
+	string(abigail::tests::get_src_dir()) + "/tests/" + s->second_in_path;
       ref_diff_path =
-	abigail::tests::get_src_dir() + "/tests/" + s->ref_diff_path;
+	string(abigail::tests::get_src_dir()) + "/tests/" + s->ref_diff_path;
       out_path =
-	abigail::tests::get_build_dir() + "/tests/" + s->out_path;
+	string(abigail::tests::get_build_dir()) + "/tests/" + s->out_path;
 
       if (!abigail::tools_utils::ensure_parent_dir_created(out_path))
 	{
@@ -189,10 +191,12 @@ main(int, char*[])
 
       translation_unit_diff_sptr d1;
       corpus_diff_sptr d2;
+      diff_context_sptr ctxt(new diff_context);
+      ctxt->show_locs(false);
       if (tu1)
-	d1= compute_diff(tu1, tu2);
+	d1= compute_diff(tu1, tu2, ctxt);
       else
-	d2 = compute_diff(corpus1, corpus2);
+	d2 = compute_diff(corpus1, corpus2, ctxt);
       ofstream of(out_path.c_str(), std::ios_base::trunc);
       if (!of.is_open())
 	{
