@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2015 Red Hat, Inc.
+// Copyright (C) 2013-2017 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -205,6 +205,126 @@ InOutSpec in_out_specs[] =
     "data/test-diff-dwarf/libtest-23-diff-arch-v0-64.so",
     "data/test-diff-dwarf/test-23-diff-arch-report-0.txt",
     "output/test-diff-dwarf/test-23-diff-arch-report-0.txt"
+  },
+    {
+    "data/test-diff-dwarf/libtest24-added-fn-parms-v0.so",
+    "data/test-diff-dwarf/libtest24-added-fn-parms-v1.so",
+    "data/test-diff-dwarf/test24-added-fn-parms-report-0.txt",
+    "output/test-diff-dwarf/test24-added-fn-parms-report-0.txt"
+  },
+  {
+    "data/test-diff-dwarf/libtest25-removed-fn-parms-v0.so",
+    "data/test-diff-dwarf/libtest25-removed-fn-parms-v1.so",
+    "data/test-diff-dwarf/test25-removed-fn-parms-report-0.txt",
+    "output/test-diff-dwarf/test25-removed-fn-parms-report-0.txt"
+  },
+  {
+    "data/test-diff-dwarf/libtest26-added-parms-before-variadic-v0.so",
+    "data/test-diff-dwarf/libtest26-added-parms-before-variadic-v1.so",
+    "data/test-diff-dwarf/test26-added-parms-before-variadic-report.txt",
+    "output/test-diff-dwarf/test26-added-parms-before-variadic-report.txt"
+  },
+  {
+    "data/test-diff-dwarf/test27-local-base-diff-v0.o",
+    "data/test-diff-dwarf/test27-local-base-diff-v1.o",
+    "data/test-diff-dwarf/test27-local-base-diff-report.txt",
+    "output/test-diff-dwarf/test27-local-base-diff-report.txt"
+  },
+  {
+    "data/test-diff-dwarf/test28-vtable-changes-v0.o",
+    "data/test-diff-dwarf/test28-vtable-changes-v1.o",
+    "data/test-diff-dwarf/test28-vtable-changes-report-0.txt",
+    "output/test-diff-dwarf/test28-vtable-changes-report-0.txt"
+  },
+  {
+    "data/test-diff-dwarf/test29-vtable-changes-v0.o",
+    "data/test-diff-dwarf/test29-vtable-changes-v1.o",
+    "data/test-diff-dwarf/test29-vtable-changes-report-0.txt",
+    "output/test-diff-dwarf/test29-vtable-changes-report-0.txt"
+  },
+  {
+    "data/test-diff-dwarf/test30-vtable-changes-v0.o",
+    "data/test-diff-dwarf/test30-vtable-changes-v1.o",
+    "data/test-diff-dwarf/test30-vtable-changes-report-0.txt",
+    "output/test-diff-dwarf/test30-vtable-changes-report-0.txt"
+  },
+  {
+    "data/test-diff-dwarf/test31-vtable-changes-v0.o",
+    "data/test-diff-dwarf/test31-vtable-changes-v1.o",
+    "data/test-diff-dwarf/test31-vtable-changes-report-0.txt",
+    "output/test-diff-dwarf/test31-vtable-changes-report-0.txt"
+  },
+  {
+    "data/test-diff-dwarf/test32-fnptr-changes-v0.o",
+    "data/test-diff-dwarf/test32-fnptr-changes-v1.o",
+    "data/test-diff-dwarf/test32-fnptr-changes-report-0.txt",
+    "output/test-diff-dwarf/test32-fnptr-changes-report-0.txt"
+  },
+  {
+    "data/test-diff-dwarf/test33-fnref-changes-v0.o",
+    "data/test-diff-dwarf/test33-fnref-changes-v1.o",
+    "data/test-diff-dwarf/test33-fnref-changes-report-0.txt",
+    "output/test-diff-dwarf/test33-fnref-changes-report-0.txt"
+  },
+  {
+    "data/test-diff-dwarf/test34-pr19173-libfoo.so",
+    "data/test-diff-dwarf/test34-pr19173-libfoo2.so",
+    "data/test-diff-dwarf/test34-pr19173-libfoo-report-0.txt",
+    "output/test-diff-dwarf/test34-pr19173-libfoo-report-0.txt"
+  },
+  {
+    "data/test-diff-dwarf/test35-pr19173-libfoo-long-gcc.so",
+    "data/test-diff-dwarf/test35-pr19173-libfoo-long-gcc2.so",
+    "data/test-diff-dwarf/test35-pr19173-libfoo-long-gcc-report-0.txt",
+    "output/test-diff-dwarf/test35-pr19173-libfoo-long-gcc-report-0.txt"
+  },
+  {
+    "data/test-diff-dwarf/test35-pr19173-libfoo-long-clang.so",
+    "data/test-diff-dwarf/test35-pr19173-libfoo-long-clang2.so",
+    "data/test-diff-dwarf/test35-pr19173-libfoo-long-clang-report-0.txt",
+    "output/test-diff-dwarf/test35-pr19173-libfoo-long-clang-report-0.txt"
+  },
+  {
+    "data/test-diff-dwarf/libtest36-ppc64-aliases-v0.so",
+    "data/test-diff-dwarf/libtest36-ppc64-aliases-v1.so",
+    "data/test-diff-dwarf/test36-ppc64-aliases-report-0.txt",
+    "output/test-diff-dwarf/test36-ppc64-aliases-report-0.txt"
+  },
+  {
+    "data/test-diff-dwarf/libtest37-union-v0.so",
+    "data/test-diff-dwarf/libtest37-union-v1.so",
+    "data/test-diff-dwarf/test37-union-report-0.txt",
+    "output/test-diff-dwarf/test37-union-report-0.txt"
+  },
+  {
+    "data/test-diff-dwarf/libtest38-union-v0.so",
+    "data/test-diff-dwarf/libtest38-union-v1.so",
+    "data/test-diff-dwarf/test38-union-report-0.txt",
+    "output/test-diff-dwarf/test38-union-report-0.txt"
+  },
+  {
+    "data/test-diff-dwarf/libtest39-union-v0.so",
+    "data/test-diff-dwarf/libtest39-union-v1.so",
+    "data/test-diff-dwarf/test39-union-report-0.txt",
+    "output/test-diff-dwarf/test39-union-report-0.txt"
+  },
+  {
+    "data/test-diff-dwarf/libtest40-v0.so",
+    "data/test-diff-dwarf/libtest40-v1.so",
+    "data/test-diff-dwarf/test40-report-0.txt",
+    "output/test-diff-dwarf/test40-report-0.txt"
+  },
+  {
+    "data/test-diff-dwarf/libtest41-PR20476-hidden-old.so",
+    "data/test-diff-dwarf/libtest41-PR20476-hidden-new.so",
+    "data/test-diff-dwarf/test41-PR20476-hidden-report-0.txt",
+    "output/test-diff-dwarf/test41-PR20476-hidden-report-0.txt"
+  },
+  {
+    "data/test-diff-dwarf/test42-PR21296-libgcc.so",
+    "data/test-diff-dwarf/test42-PR21296-libclang.so",
+    "data/test-diff-dwarf/test42-PR21296-clanggcc-report0.txt",
+    "output/test-diff-dwarf/test42-PR21296-clanggcc-report0.txt"
   },
   // This should be the last entry
   {NULL, NULL, NULL, NULL}
@@ -219,17 +339,21 @@ main()
   using abigail::dwarf_reader::read_corpus_from_elf;
   using abigail::comparison::compute_diff;
   using abigail::comparison::corpus_diff_sptr;
+  using abigail::ir::environment;
+  using abigail::ir::environment_sptr;
+  using abigail::comparison::diff_context_sptr;
+  using abigail::comparison::diff_context;
 
   bool is_ok = true;
   string in_elfv0_path, in_elfv1_path,
     ref_diff_report_path, out_diff_report_path;
-  abigail::corpus_sptr corp0, corp1;
 
   for (InOutSpec* s = in_out_specs; s->in_elfv0_path; ++s)
     {
-      in_elfv0_path = get_src_dir() + "/tests/" + s->in_elfv0_path;
-      in_elfv1_path = get_src_dir() + "/tests/" + s->in_elfv1_path;
-      out_diff_report_path = get_build_dir() + "/tests/" + s->out_report_path;
+      in_elfv0_path = string(get_src_dir()) + "/tests/" + s->in_elfv0_path;
+      in_elfv1_path = string(get_src_dir()) + "/tests/" + s->in_elfv1_path;
+      out_diff_report_path =
+	string(get_build_dir()) + "/tests/" + s->out_report_path;
 
       if (!ensure_parent_dir_created(out_diff_report_path))
 	{
@@ -239,12 +363,23 @@ main()
 	  continue;
 	}
 
-      read_corpus_from_elf(in_elfv0_path,
-			   /*debug_info_root_path=*/0,
-			   corp0);
-      read_corpus_from_elf(in_elfv1_path,
-			   /*debug_info_root_path=*/0,
-			   corp1);
+      abigail::dwarf_reader::status status =
+	abigail::dwarf_reader::STATUS_UNKNOWN;
+
+      environment_sptr env(new environment);
+      abigail::corpus_sptr corp0 =
+	read_corpus_from_elf(in_elfv0_path,
+			     /*debug_info_root_path=*/0,
+			     env.get(),
+			     /*load_all_types=*/false,
+			     status);
+
+      abigail::corpus_sptr corp1 =
+	read_corpus_from_elf(in_elfv1_path,
+			     /*debug_info_root_path=*/0,
+			     env.get(),
+			     /*load_all_types=*/false,
+			     status);
 
       if (!corp0)
 	{
@@ -252,6 +387,7 @@ main()
 	  is_ok = false;
 	  continue;
 	}
+
       if (!corp1)
 	{
 	  cerr << "failed to read " << in_elfv1_path << "\n";
@@ -262,7 +398,10 @@ main()
       corp0->set_path(s->in_elfv0_path);
       corp1->set_path(s->in_elfv1_path);
 
-      corpus_diff_sptr d = compute_diff(corp0, corp1);
+      diff_context_sptr ctxt(new diff_context);
+      ctxt->show_locs(false);
+
+      corpus_diff_sptr d = compute_diff(corp0, corp1, ctxt);
       if (!d)
 	{
 	  cerr << "failed to compute diff\n";
@@ -270,8 +409,10 @@ main()
 	  continue;
 	}
 
-      ref_diff_report_path = get_src_dir() + "/tests/" + s->in_report_path;
-      out_diff_report_path = get_build_dir() + "/tests/" + s->out_report_path;
+      ref_diff_report_path =
+	string(get_src_dir()) + "/tests/" + s->in_report_path;
+      out_diff_report_path =
+	string(get_build_dir()) + "/tests/" + s->out_report_path;
 
       ofstream of(out_diff_report_path.c_str(), std::ios_base::trunc);
       if (!of.is_open())
